(*************************************************
TpvlOvcTCCalcEdit is the link of a pvlCalcedit to
an Orpheus Table Component.

This Source Code remains Copyrighted to
CopyRight (c) 1998,99 PA van Lonkhuyzen

It may be distributed/modified/used freely.

The Author may be contacted via
email peterv@global.co.za
post P A van Lonkhuyzen
     PO Box 1041
     Randpark Ridge
     2156
     South Africa
***************************************************)
{D-,L-} {Remove if you wish to debug this code}
unit CalcCell;
interface

uses
{$ifdef win32}
  Windows,
{$else}
WinProcs,
Wintypes,
{$endif}  SysUtils, Messages,
  Graphics, Classes, Controls, Forms, StdCtrls,
  OvcData, OvcConst, OvcBase, OvcMisc,EdTool,
  OvcTCmmn, OvcTCell, OvcTGPns, OvcTable, OvcTCStr;

type
  TpvlOvcCalcEditEdit = class(TpvlCustomCalcEdit)
    protected
      FCell     : TOvcBaseTableCell;
      EditField : HWnd;
    protected
      procedure WMChar(var Msg : TWMKey); message WM_CHAR;
      procedure WMGetDlgCode(var Msg : TMessage); message WM_GETDLGCODE;
      procedure WMKeyDown(var Msg : TWMKey); message WM_KEYDOWN;
      procedure WMSetFocus(var Msg : TWMSetFocus); message WM_SETFOCUS;
    public
      property CellOwner : TOvcBaseTableCell read FCell write FCell;
  end;


  TpvlOvcTCCustomCalcEdit = class(TOvcTCBaseString)
  private
    fDontWantFormEvent: TWantFormEvent;
    protected {private}
      fdec : byte;
      fWantFormEvent : TWantFormEvent;
      fAlignment : TCalcAlignment;
      fSubAlign : TSubAlign;
      FMaxLength     : word;
      FAutoAdvanceChar      : boolean;
      FAutoAdvanceLeftRight : boolean;
      FOnChange      : TNotifyEvent;
      fDisplayFormat : TCalcDisplayFormat;
      fDisplayFormatMask : String;
      fminmaxhandler : TMinMaxhandler;
      fOnMinmaxError : TMinMaxEvent;
      fMinMaxString : String;
      fmin,
      fmax : Extended;
      FOnDoneValue : TNotifyEvent;
    protected
      function GetCellEditor : TControl; override;
      procedure tcPaint(TableCanvas : TCanvas;
                  const CellRect    : TRect;
                        RowNum      : TRowNum;
                        ColNum      : TColNum;
                  const CellAttr    : TOvcCellAttributes;
                        Data        : pointer); override;
      property Alignment : TCalcAlignment read falignment write falignment;
      property AutoAdvanceChar : boolean read FAutoAdvanceChar write FAutoAdvanceChar;
      property AutoAdvanceLeftRight : boolean read FAutoAdvanceLeftRight write FAutoAdvanceLeftRight;
      property SubAlign : TSubalign read fsubalign write fsubalign;
      property MaxLength : word read FMaxLength write FMaxLength;
      property OnChange : TNotifyEvent read FOnChange write FOnChange;
      Property DisplayFormat : TCalcDisplayFormat Read fDisplayFormat Write fDisplayFormat Default cfNumber;
      Property DisplayFormatMask : String Read fDisplayFormatMask Write fDisplayFormatMask;
      Property Min : Extended Read fmin Write fMin;
      Property Max : Extended Read fmax Write fMax;
      Property OnDoneValue : TNotifyEvent read fOnDoneValue write fOnDoneValue;
      Property MinMaxHandler : TMinMaxHandler read fMinMaxHandler write fMinMaxHandler;
      Property OnMinmaxError : TMinMaxEvent  Read fOnMinMaxError write fOnMinMaxError;
      Property MinMaxString : String read fMinMaxString write fMinMaxString;
      Property OnWantForm : TWantFormEvent Read fWantFormEvent Write fWantFormEvent;
      Property OnBeforeSubFormFree : TWantFormEvent Read fDontWantFormEvent Write fDontWantFormEvent;
     public
      FEditor          : TpvlOvcCalcEditEdit;
      property Decimals : byte read fdec write fdec default 2;
      constructor Create(AOwner : TComponent); override;
      destructor Destroy; override;
      function  EditHandle : THandle; override;
      procedure EditHide; override;
      procedure EditMove(CellRect : TRect); override;
      procedure SaveEditedData(Data : pointer); override;
      procedure StartEditing(RowNum : TRowNum; ColNum : TColNum;
                             CellRect : TRect;
                       const CellAttr : TOvcCellAttributes;
                             CellStyle: TOvcTblEditorStyle;
                             Data : pointer); override;
      procedure StopEditing(SaveValue : boolean;
                            Data : pointer); override;
  end;

  TpvlOvcTCCalcEdit = class(TpvlOvcTCCustomCalcEdit)
    published
      Property OnWantForm;
      Property Alignment;
      Property DisplayFormat;
      Property DisplayFormatMask;
      property Decimals;
      property AcceptActivationClick;
      property Access;
      property Adjust;
      property AutoAdvanceChar;
      property AutoAdvanceLeftRight;
      property Color;
      property Font;
      property Hint;
      property ShowHint;
      property Margin;
      property MaxLength;
      Property SubAlign;
      property Table;
      property TableColor;
      property TableFont;
      property TextHiColor;
      property TextStyle;
      property OnChange;
      property OnClick;
      property OnDblClick;
      property OnDragDrop;
      property OnDragOver;
      property OnEndDrag;
      property OnEnter;
      property OnExit;
      property OnKeyDown;
      property OnKeyPress;
      property OnKeyUp;
      property OnMouseDown;
      property OnMouseMove;
      property OnMouseUp;
      property OnOwnerDraw;
      Property Min;
      Property Max;
      Property OnDoneValue;
      Property MinMaxHandler;
      Property OnMinmaxError;
      Property MinMaxString;
      Property OnBeforeSubFormFree;
  end;

implementation

{===TpvlOvcCalcEditEdit================================================}
{--------}
procedure TpvlOvcCalcEditEdit.WMChar(var Msg : TWMKey);
var
  CurText : string;
begin
  inherited;
  if TpvlOvcTCCustomCalcEdit(CellOwner).AutoAdvanceChar then
  begin
    CurText := Text;
    if (length(CurText) >= MaxLength) then
    begin
      FillChar(Msg, sizeof(Msg), 0);
      with Msg do
      begin
        Msg := WM_KEYDOWN;
        CharCode := VK_RIGHT;
      end;
      CellOwner.SendKeyToTable(Msg);
    end;
   end;
 end;
{--------}
procedure TpvlOvcCalcEditEdit.WMGetDlgCode(var Msg : TMessage);
begin
  inherited;
  if CellOwner.TableWantsTab then
      Msg.Result := Msg.Result or DLGC_WANTTAB;
end;
{--------}
procedure TpvlOvcCalcEditEdit.WMKeyDown(var Msg : TWMKey);
var
    GridReply    : TOvcTblKeyNeeds;
    GridUsedIt   : boolean;
begin
  GridUsedIt := false;
  GridReply := otkDontCare;
  if (CellOwner <> nil) then
    GridReply := CellOwner.FilterTableKey(Msg);
  case GridReply of
    otkMustHave : GridUsedIt := true;
    otkWouldLike : begin
          gridusedit := not Button.enabled;
          if not gridusedit then
            case msg.CharCode of
    vk_delete : begin
              if (text='') or ((seltext=decimalseparator) and (pos(decimalseparator,text)<>length(text))) then
              begin
                gridusedit:=true;
              end
              else
              if seltext='' then
              begin
                if (text[selstart+1]=decimalseparator) and (pos(decimalseparator,text)<>length(text)) then
                begin
                   gridusedit:=true;
                end;
              end;
              end;
    vk_left : Gridusedit:=(selstart=0) and (sellength=0);
    vk_right : GridUsedIt := (SelStart>=length(text)) and (sellength=0);
    vk_home:;
    vk_end:;
    ord(0)..ord(9),ord('+'),ord('-'),ord('*'),ord('/') :;
            else
              gridusedit:=true;
           end;
        end;
  end;
  if GridUsedIt then
    CellOwner.SendKeyToTable(Msg)
  else
    inherited;
end;
{--------}
procedure TpvlOvcCalcEditEdit.WMSetFocus(var Msg : TWMSetFocus);
begin
  inherited;
  CellOwner.PostMessageToTable(ctim_SetFocus, Msg.FocusedWnd, 0);
end;
{====================================================================}


{===TpvlOvcTCCustomCalcEdit=============================================}
constructor TpvlOvcTCCustomCalcEdit.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);
  fmaxlength:=13;
  fdec:=2;
  FAcceptActivationClick := true;
  fDisplayFormat:=cfNumber;
end;
{--------}
destructor TpvlOvcTCCustomCalcEdit.Destroy;
  begin
    inherited Destroy;
  end;
{--------}
function TpvlOvcTCCustomCalcEdit.EditHandle : THandle;
begin
  if Assigned(FEditor) then
    Result := FEditor.Handle
  else
    Result := 0;
 end;
{--------}
procedure TpvlOvcTCCustomCalcEdit.EditHide;
begin
  if Assigned(FEditor) then
    FEditor.visible:=false;
end;
{--------}
procedure TpvlOvcTCCustomCalcEdit.EditMove(CellRect : TRect);
begin
  FEditor.boundsrect:=Cellrect;
  FEditor.visible:=True;
end;
{--------}
function TpvlOvcTCCustomCalcEdit.GetCellEditor : TControl;
begin
  Result := FEditor;
end;
{--------}
procedure TpvlOvcTCCustomCalcEdit.tcPaint(TableCanvas : TCanvas;
                                 const CellRect    : TRect;
                                       RowNum      : TRowNum;
                                       ColNum      : TColNum;
                                 const CellAttr    : TOvcCellAttributes;
                                       Data        : pointer);
  var
    Value : PExtended absolute Data;
    S         : ShortString;
  begin
    {If the cell is invisible let the ancestor to all the work}
    if (CellAttr.caAccess = otxInvisible) then
      begin
        inherited tcPaint(TableCanvas, CellRect, RowNum, ColNum, CellAttr, nil);
        Exit;
      end;
    S := '';
    if (Data <> nil) then
      begin
         case fDisplayFormat of
          cfnumber : s:=format('%.'+inttostr(fdec)+'n',[value^]);
          else
            s:=format(fdisplayformatmask,[value^]);
         end;
      end
    else if (csDesigning in ComponentState) then
      S := 'Calc';
    inherited tcPaint(TableCanvas, CellRect, RowNum, ColNum, CellAttr, @s);

  end;
{--------}
procedure TpvlOvcTCCustomCalcEdit.SaveEditedData(Data : pointer);
  var
    Value : PExtended absolute Data;
  begin
    if Assigned(Data) then
      begin
        Value^ := FEditor.Value;
      end;
  end;
{--------}
{--------}
procedure TpvlOvcTCCustomCalcEdit.StartEditing(RowNum : TRowNum; ColNum : TColNum;
                                           CellRect : TRect;
                                     const CellAttr : TOvcCellAttributes;
                                           CellStyle: TOvcTblEditorStyle;
                                           Data : pointer);
  var
    OrgValue : PExtended absolute Data;
  begin
    FEditor := TpvlOvcCalcEditEdit.Create(FTable);
    with FEditor do
      begin
        OnDoneValue:=FOnDoneValue;
        MinMaxHandler:=fMinMaxHandler;
        OnMinmaxError:=fOnMinMaxError;
        MinMaxString:=fMinMaxString;
        SubAlign:=fSubAlign;
        Displayformat:=fDisplayformat;
        decimals:=fdec;
        Displayformatmask:=fDisplayFormatMask;
        Color := CellAttr.caColor;
        Ctl3D := false;
        case CellStyle of
          tes3D     : Ctl3D := true;
        end;{case}
        Alignment:=fAlignment;
        FEditor.boundsrect:=CellRect;
        Font := CellAttr.caFont;
        Font.Color := CellAttr.caFontColor;
        MaxLength := fMaxLength;
        Hint := fHint;
        ShowHint := fShowHint;
        Visible := true;
        CellOwner := Self;
        TabStop := false;
        Parent := FTable;
        Value:=OrgValue^;
        Onwantform:=self.onwantform;
        OnBeforeSubFormFree := self.OnBeforeSubFormFree;
        OnChange := Self.OnChange;
        OnClick := Self.OnClick;
        OnDblClick := Self.OnDblClick;
        OnDragDrop := Self.OnDragDrop;
        OnDragOver := Self.OnDragOver;
        OnEndDrag := Self.OnEndDrag;
        OnEnter := Self.OnEnter;
        OnExit := Self.OnExit;
        OnKeyDown := Self.OnKeyDown;
        OnKeyPress := Self.OnKeyPress;
        OnKeyUp := Self.OnKeyUp;
        OnMouseDown := Self.OnMouseDown;
        OnMouseMove := Self.OnMouseMove;
        OnMouseUp := Self.OnMouseUp;
        Max:=fmax;
        Min:=fmin;
      end;
  end;
{--------}
procedure TpvlOvcTCCustomCalcEdit.StopEditing(SaveValue : boolean;
                                          Data : pointer);
  var
    Value :PExtended absolute Data;
  begin
    if SaveValue and Assigned(Data) then
      begin
        Value^ := FEditor.Value;
      end;
    FEditor.free;
    FEditor := nil;
  end;
{====================================================================}

end.
