(*************************************************
TpvlOvcTCTimeEdit is the link of a pvlTimeedit to
an Orpheus Table Component.

This Source Code remains Copyrighted to
CopyRight (c) 1998 PA van Lonkhuyzen

It may be distributed/modified/used freely.

The Author may be contacted via
email peterv@global.co.za
post P A van Lonkhuyzen
     PO Box 1041
     Randpark Ridge
     2156
     South Africa
***************************************************)
{$D-,L-} {Remove if you wish to debug this code}
unit TimeCell;
interface

uses
{$ifdef win32}
  Windows,
{$else}
WinProcs,
Wintypes,
{$endif}  SysUtils, Messages,
  Graphics, Classes, Controls, Forms, StdCtrls,
  OvcData, OvcConst, OvcBase, OvcMisc,EdTool,pvldedit,
  OvcTCmmn, OvcTCell, OvcTGPns, OvcTable, OvcTCStr;

type
  TpvlOvcTimeEditEdit = class(TpvlCustomTimeEdit)
    protected
      FCell     : TOvcBaseTableCell;
      EditField : HWnd;
    protected
      procedure WMChar(var Msg : TWMKey); message WM_CHAR;
      procedure WMGetDlgCode(var Msg : TMessage); message WM_GETDLGCODE;
      procedure WMKeyDown(var Msg : TWMKey); message WM_KEYDOWN;
      procedure WMSetFocus(var Msg : TWMSetFocus); message WM_SETFOCUS;
    public
      property CellOwner : TOvcBaseTableCell read FCell write FCell;
  end;


  TpvlOvcTCCustomTimeEdit = class(TOvcTCBaseString)
  private
    fDontWantFormEvent: TWantFormEvent;
    protected {private}
      fSeparateHourHand : Boolean;
      fAMPMHeadings : TStrings;
      fQuickChangeType : TQuickChangeType;
      fQuickChangeWrap : Boolean;
      fWantFormEvent : TWantFormEvent;
      fSubAlign : TSubAlign;
      fEditFormat : TTimeEditFormat;
      FEditor          : TpvlOvcTimeEditEdit;
      FAutoAdvanceChar      : boolean;
      FAutoAdvanceLeftRight : boolean;
      FOnChange      : TNotifyEvent;
      fDisplayFormat : TDateDisplayFormat;
      fDisplayFormatMask : String;
      FOnTimeChange : TNotifyEvent;
      procedure loaded; override;
      function GetCellEditor : TControl; override;
      procedure tcPaint(TableCanvas : TCanvas;
                  const CellRect    : TRect;
                        RowNum      : TRowNum;
                        ColNum      : TColNum;
                  const CellAttr    : TOvcCellAttributes;
                        Data        : pointer); override;
      procedure SetAMPMHeadings(s: TStrings);
      property AutoAdvanceChar : boolean read FAutoAdvanceChar write FAutoAdvanceChar;
      property AutoAdvanceLeftRight : boolean read FAutoAdvanceLeftRight write FAutoAdvanceLeftRight;
      property SubAlign : TSubalign read fsubalign write fsubalign;
      property OnChange : TNotifyEvent read FOnChange write FOnChange;
      Property DisplayFormat : TDateDisplayFormat Read fDisplayFormat Write fDisplayFormat Default dfLong;
      Property DisplayFormatMask : String Read fDisplayFormatMask Write fDisplayFormatMask;
      Property OnWantForm : TWantFormEvent Read fWantFormEvent Write fWantFormEvent;
      Property AMPMHeadings : TStrings read fAMPMHeadings write SetAMPMHeadings;
      property QuickChangeType : TQuickChangeType read fQuickChangeType write fQuickChangeType;
      property QuickChangeWrap : Boolean read fQuickChangeWrap write fQuickChangeWrap;
      Property EditFormat : TTimeEditFormat Read fEditFormat Write fEditFormat Default teExcludeSecs;
      Property OnTimeChange : TNotifyEvent Read fOnTimeChange Write fOnTimeChange;
      property SeparateHourHand : Boolean read fSeparateHourHand write fSeparateHourHand;
      Property OnBeforeSubFormFree : TWantFormEvent Read fDontWantFormEvent Write fDontWantFormEvent;
     public
      constructor Create(AOwner : TComponent); override;
      destructor Destroy; override;
      function  EditHandle : THandle; override;
      procedure EditHide; override;
      procedure EditMove(CellRect : TRect); override;
      procedure SaveEditedData(Data : pointer); override;
      procedure StartEditing(RowNum : TRowNum; ColNum : TColNum;
                             CellRect : TRect;
                       const CellAttr : TOvcCellAttributes;
                             CellStyle: TOvcTblEditorStyle;
                             Data : pointer); override;
      procedure StopEditing(SaveValue : boolean;
                            Data : pointer); override;
  end;

  TpvlOvcTCTimeEdit = class(TpvlOvcTCCustomTimeEdit)
  public
  published
      Property OnTimeChange;
      Property AMPMHeadings;
      Property OnWantForm;
      Property DisplayFormat;
      Property DisplayFormatMask;
      property AcceptActivationClick;
      property Access;
      property Adjust;
      property AutoAdvanceChar;
      property AutoAdvanceLeftRight;
      property Color;
      property Font;
      property Hint;
      property ShowHint;
      property Margin;
      Property SubAlign;
      property Table;
      property TableColor;
      property TableFont;
      property TextHiColor;
      property TextStyle;
      property OnChange;
      property OnClick;
      property OnDblClick;
      property OnDragDrop;
      property OnDragOver;
      property OnEndDrag;
      property OnEnter;
      property OnExit;
      property OnKeyDown;
      property OnKeyPress;
      property OnKeyUp;
      property OnMouseDown;
      property OnMouseMove;
      property OnMouseUp;
      property OnOwnerDraw;
      property QuickChangeType;
      property QuickChangeWrap;
      Property EditFormat;
      property SeparateHourHand;
      Property OnBeforeSubFormFree;

  end;

implementation
Type PDatetime = ^tDatetime;
{===TpvlOvcTimeEditEdit================================================}
{--------}
procedure TpvlOvcTimeEditEdit.WMChar(var Msg : TWMKey);
var
  CurText : string;
begin
  inherited;
  if TpvlOvcTCCustomTimeEdit(CellOwner).AutoAdvanceChar then
  begin
    CurText := Text;
    if (length(CurText) >= MaxLength) then
    begin
      FillChar(Msg, sizeof(Msg), 0);
      with Msg do
      begin
        Msg := WM_KEYDOWN;
        CharCode := VK_RIGHT;
      end;
      CellOwner.SendKeyToTable(Msg);
    end;
   end;
 end;
{--------}
procedure TpvlOvcTimeEditEdit.WMGetDlgCode(var Msg : TMessage);
begin
  inherited;
  if CellOwner.TableWantsTab then
      Msg.Result := Msg.Result or DLGC_WANTTAB;
end;
{--------}
procedure TpvlOvcTimeEditEdit.WMKeyDown(var Msg : TWMKey);
var
    GridReply    : TOvcTblKeyNeeds;
    GridUsedIt   : boolean;
begin
  GridUsedIt := false;
  GridReply := otkDontCare;
  if (CellOwner <> nil) then
    GridReply := CellOwner.FilterTableKey(Msg);
  case GridReply of
    otkMustHave : GridUsedIt := true;
    otkWouldLike : begin
          gridusedit := not Button.enabled;
          if not gridusedit then
            case msg.CharCode of
    vk_delete : ;
    vk_left : Gridusedit:=(getselstart=0) and (getsellength=0);
    vk_right : GridUsedIt := (GetSelStart>=length(text)) and (getsellength=0);
    vk_home:;
    vk_end:;
    ord('+'),ord('-') : gridusedit:=(QuickChangeType=qcNone);
    ord(0)..ord(9):;
            else
              gridusedit:=true;
           end;
        end;
  end;
  if GridUsedIt then
    CellOwner.SendKeyToTable(Msg)
  else
    inherited;
end;
{--------}
procedure TpvlOvcTimeEditEdit.WMSetFocus(var Msg : TWMSetFocus);
begin
  inherited;
  CellOwner.PostMessageToTable(ctim_SetFocus, Msg.FocusedWnd, 0);
end;
{====================================================================}


{===TpvlOvcTCCustomTimeEdit=============================================}


procedure TPvlOvcTcCustomTimeEdit.SetAMPMHeadings(s: TStrings);
begin
  fAMPMHeadings.Assign(s);
end;

procedure TPvlOvcTcCustomTimeEdit.Loaded;
begin
  inherited Loaded;
end;

constructor TpvlOvcTCCustomTimeEdit.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);
  FAcceptActivationClick := true;
  fDisplayFormat:=dflong;
  feditformat:=teExcludeSecs;
  fAMPMHeadings:=Tstringlist.create;
end;
{--------}
destructor TpvlOvcTCCustomTimeEdit.Destroy;
begin
  fAMPMHeadings.free;
  inherited Destroy;
end;
{--------}

function TpvlOvcTCCustomTimeEdit.EditHandle : THandle;
begin
  if Assigned(FEditor) then
    Result := FEditor.Handle
  else
    Result := 0;
 end;
{--------}
procedure TpvlOvcTCCustomTimeEdit.EditHide;
begin
  if Assigned(FEditor) then
    FEditor.visible:=false;
end;
{--------}
procedure TpvlOvcTCCustomTimeEdit.EditMove(CellRect : TRect);
begin
  FEditor.boundsrect:=Cellrect;
  FEditor.visible:=True;
end;
{--------}
function TpvlOvcTCCustomTimeEdit.GetCellEditor : TControl;
begin
  Result := FEditor;
end;
{--------}
procedure TpvlOvcTCCustomTimeEdit.tcPaint(TableCanvas : TCanvas;
                                 const CellRect    : TRect;
                                       RowNum      : TRowNum;
                                       ColNum      : TColNum;
                                 const CellAttr    : TOvcCellAttributes;
                                       Data        : pointer);
  var
    Value : PDatetime absolute Data;
    S         : ShortString;
  begin
    {If the cell is invisible let the ancestor to all the work}
    if (CellAttr.caAccess = otxInvisible) then
      begin
        inherited tcPaint(TableCanvas, CellRect, RowNum, ColNum, CellAttr, nil);
        Exit;
      end;
    S := '';
    if (Data <> nil) then
        case fDisplayFormat of
         dfshort : s:=formatDatetime('t',value^);
         dflong : s:=formatDatetime('tt',value^);
         else
           s:=formatDatetime(fdisplayformatmask,value^);
        end
    else if (csDesigning in ComponentState) then
      S := 'Time';
    inherited tcPaint(TableCanvas, CellRect, RowNum, ColNum, CellAttr, @s);

  end;
{--------}
procedure TpvlOvcTCCustomTimeEdit.SaveEditedData(Data : pointer);
  var
    Value : PDatetime absolute Data;
  begin
    if Assigned(Data) then
      begin
        Value^ := FEditor.Time;
      end;
  end;
{--------}
{--------}
procedure TpvlOvcTCCustomTimeEdit.StartEditing(RowNum : TRowNum; ColNum : TColNum;
                                           CellRect : TRect;
                                     const CellAttr : TOvcCellAttributes;
                                           CellStyle: TOvcTblEditorStyle;
                                           Data : pointer);
  var
    OrgValue : PDatetime absolute Data;
  begin
    FEditor := TpvlOvcTimeEditEdit.Create(FTable);
    with FEditor do
      begin
        OnTimeChange:=fOnTimeChange;
        QuickChangeType:=fQuickChangeType;
        QuickChangeWrap:=fQuickChangeWrap;
        SeparateHourHand := fSeparateHourHand;
        EditFormat:=fEditFormat;
        AMPMHeadings:=fAMPMHeadings;
        SubAlign:=fSubAlign;
        Displayformat:=fDisplayformat;
        Displayformatmask:=fDisplayFormatMask;
        Color := CellAttr.caColor;
        Ctl3D := false;
        case CellStyle of
          tes3D     : Ctl3D := true;
        end;{case}
        FEditor.boundsrect:=CellRect;
        Font := CellAttr.caFont;
        Font.Color := CellAttr.caFontColor;
        Hint := fHint;
        ShowHint := fShowHint;
        Visible := true;
        CellOwner := Self;
        TabStop := false;
        Parent := FTable;
        Time:=OrgValue^;
        Onwantform:=self.onwantform;
        OnBeforeSubFormFree := self.OnBeforeSubFormFree;
        OnChange := Self.OnChange;
        OnClick := Self.OnClick;
        OnDblClick := Self.OnDblClick;
        OnDragDrop := Self.OnDragDrop;
        OnDragOver := Self.OnDragOver;
        OnEndDrag := Self.OnEndDrag;
        OnEnter := Self.OnEnter;
        OnExit := Self.OnExit;
        OnKeyDown := Self.OnKeyDown;
        OnKeyPress := Self.OnKeyPress;
        OnKeyUp := Self.OnKeyUp;
        OnMouseDown := Self.OnMouseDown;
        OnMouseMove := Self.OnMouseMove;
        OnMouseUp := Self.OnMouseUp;
      end;
  end;
{--------}
procedure TpvlOvcTCCustomTimeEdit.StopEditing(SaveValue : boolean;
                                          Data : pointer);
  var
    Value :PDatetime absolute Data;
  begin
    if SaveValue and Assigned(Data) then
      begin
        Value^ := FEditor.Time;
      end;
    FEditor.free;
    FEditor := nil;
  end;
{====================================================================}

end.
